@@ -1,7 +1,7 @@
 package Perinci::Sub::Wrapper;
 
-our $DATE = '2015-02-18'; # DATE
-our $VERSION = '0.73'; # VERSION
+our $DATE = '2015-02-19'; # DATE
+our $VERSION = '0.74'; # VERSION
 
 use 5.010001;
 use strict;
@@ -49,8 +49,12 @@ sub _add_module {
     unless ($mod ~~ $self->{_modules}) {
         local $self->{_cur_section};
         $self->select_section('before_sub_require_modules');
-        if ($mod =~ /(.+?)\s+(.+)/) {
-            $self->push_lines("use $1 $2;");
+        if ($mod =~ /(-?)(.+?)\s+(.+)/) {
+            if ($1) {
+                $self->push_lines("no $2 $3;");
+            } else {
+                $self->push_lines("use $2 $3;");
+            }
         } else {
             $self->push_lines("require $mod;");
         }
@@ -58,6 +62,22 @@ sub _add_module {
     }
 }
 
+sub _add_modules {
+    my ($self, $mods) = @_;
+    for my $mod (sort keys %$mods) {
+        my $modspec = $mods->{$mod};
+        if ($modspec->[1]) {
+            if ($modspec->[0] eq 'no') {
+                $self->_add_module("-$mod ".join(" ", @{ $modspec->[1] }));
+            } else {
+                $self->_add_module( "$mod ".join(" ", @{ $modspec->[1] }));
+            }
+        } else {
+            $self->_add_module($mod);
+        }
+    }
+}
+
 sub _add_var {
     my ($self, $var, $value) = @_;
     unless (exists $self->{_vars}{$var}) {
@@ -616,7 +636,8 @@ sub _handle_args {
                     indent_level         => $self->get_indent_level + 1,
                     %{ $self->{_args}{_extra_sah_compiler_args} // {}},
                 );
-                $self->_add_module($_) for @{ $cd->{modules} };
+                $self->_add_modules($cd->{module_statements});
+                for (@{ $cd->{modules} }) { $self->_add_module($_) unless $cd->{module_statements}{$_} }
                 $self->_add_var($_, $cd->{vars}{$_})
                     for sort keys %{ $cd->{vars} };
                 $cd->{result} =~ s/\A\s+//;
@@ -805,7 +826,8 @@ sub handle_result {
                 indent_level         => $self->get_indent_level + 1,
                 %{ $self->{_args}{_extra_sah_compiler_args} // {}},
             );
-            $self->_add_module($_) for @{ $cd->{modules} };
+            $self->_add_modules($cd->{module_statements});
+            for (@{ $cd->{modules} }) { $self->_add_module($_) unless $cd->{module_statements}{$_} }
             $self->_add_var($_, $cd->{vars}{$_})
                 for sort keys %{ $cd->{vars} };
             $self->push_lines("my \$_w_err2_res;");
@@ -911,6 +933,7 @@ sub handle_deps {
 
 sub handlemeta_x { {} }
 sub handlemeta_entity_v { {} }
+sub handlemeta_entity_date { {} }
 
 sub _reset_work_data {
     my ($self, %args) = @_;
@@ -1343,7 +1366,7 @@ Perinci::Sub::Wrapper - A multi-purpose subroutine wrapping framework
 
 =head1 VERSION
 
-This document describes version 0.73 of Perinci::Sub::Wrapper (from Perl distribution Perinci-Sub-Wrapper), released on 2015-02-18.
+This document describes version 0.74 of Perinci::Sub::Wrapper (from Perl distribution Perinci-Sub-Wrapper), released on 2015-02-19.
 
 =head1 SYNOPSIS
 
